@@ -336,13 +336,45 @@ H5Pget_char_encoding <- function( h5plist ) {
   res
 }
 
-H5Pset_create_intermediate_group <- function( h5plist, crt_intermed_group ) {
+#' Get and set whether to create missing intermediate groups
+#'
+#' @param h5plist An object of class [H5IdComponent-class] representing a link
+#'   creation property list.
+#' @param create_groups A logical of length 1 specifying whether missing
+#'   groups should be created when a new object is created.  Default is `TRUE`.
+#'   
+#' @examples  
+#' pid <- H5Pcreate("H5P_LINK_CREATE")
+#' 
+#' ## by default intermediate groups are not created
+#' H5Pget_create_intermediate_group( pid )
+#' 
+#' ## Change the setting so groups will be created
+#' 
+#' H5Pget_create_intermediate_group( pid )
+#' 
+#' ## tidy up
+#' H5Pclose(pid)
+#'
+#' @name H5P_create_intermediate_group
+NULL
+
+#' @rdname H5P_create_intermediate_group 
+#' @export
+H5Pset_create_intermediate_group <- function( h5plist, create_groups = TRUE ) {
+  
   h5checktypeAndPLC(h5plist, "H5P_LINK_CREATE")
-  crt_intermed_group = as.integer(crt_intermed_group)
+  if(!is.logical(create_groups)) {
+    stop("The 'create_groups' argument should be either TRUE or FALSE")
+  }
+  
+  crt_intermed_group = as.integer(create_groups)
   res <- .Call("_H5Pset_create_intermediate_group", h5plist@ID, crt_intermed_group, PACKAGE='rhdf5')
   invisible(res)
 }
 
+#' @rdname H5P_create_intermediate_group 
+#' @export
 H5Pget_create_intermediate_group <- function( h5plist ) {
   h5checktypeAndPLC(h5plist, "H5P_LINK_CREATE")
   res <- .Call("_H5Pget_create_intermediate_group", h5plist@ID, PACKAGE='rhdf5')
@@ -445,7 +477,7 @@ H5Pset_deflate <- function( h5plist, level ) {
 #'
 #' @param h5plist An object of class [H5IdComponent-class] representing a
 #'   dataset creation property list.
-#' @param value The default fill value of the dataset.
+#' @param value The default fill value of the dataset. A vector of length 1.
 #' 
 #' @seealso [H5P_fill_time],[H5Pfill_value_defined]
 #'
@@ -455,6 +487,11 @@ NULL
 #' @rdname H5P_fill_value
 #' @export
 H5Pset_fill_value <- function( h5plist, value ) {
+  
+  if(length(value) > 1L) {
+    stop("'value' must be a vector of length 1.")
+  }
+  
   h5checktypeAndPLC(h5plist, "H5P_DATASET_CREATE")
   storage.mode = storage.mode(value)
   tid <- switch(storage.mode,
@@ -463,8 +500,11 @@ H5Pset_fill_value <- function( h5plist, value ) {
                 logical = h5constants$H5T["H5T_STD_I8LE"],
                 character = {
                     tid <- H5Tcopy("H5T_C_S1")
-                    size <- nchar(value)+1
+                    size <- nchar(value, type = "bytes")
                     H5Tset_size(tid, size)
+                    H5Tset_strpad(tid, strpad = "NULLPAD")
+                    if(Encoding(value) == "UTF-8") { cset <- "UTF-8" } else { cset <- "ASCII" }
+                    H5Tset_cset(tid, cset = cset)
                     tid
                 },
                 { stop("datatype ",storage.mode," not supported. Try 'double', 'integer', or 'character'.") } )
